@@ -16,13 +16,18 @@ class LSDynaLicenseServer(LicenseServerInterface):
         self.license_servers = license_servers
         self.parser = lsdyna.parse
 
-    def get_commands_list(self):
+    def get_commands_list(self) -> typing.List[typing.List[str]]:
         """Generate a list of commands with the available license server hosts."""
 
         host_ports = [(server.split(":")[1:]) for server in self.license_servers]
         commands_to_run = []
         for host, port in host_ports:
-            command_line = f"{settings.LSDYNA_PATH} -s {port}@{host} -R"
+            command_line = [
+                f"{settings.LSDYNA_PATH}",
+                "-s",
+                f"{port}@{host}",
+                "-R",
+            ]
             commands_to_run.append(command_line)
         return commands_to_run
 
