@@ -16,13 +16,17 @@ class OLicenseLicenseServer(LicenseServerInterface):
         self.license_servers = license_servers
         self.parser = olicense.parse
 
-    def get_commands_list(self) -> typing.List[str]:
+    def get_commands_list(self) -> typing.List[typing.List[str]]:
         """Generate a list of commands with the available license server hosts."""
 
         host_ports = [(server.split(":")[1:]) for server in self.license_servers]
         commands_to_run = []
         for host, port in host_ports:
-            command_line = f"{settings.OLIXTOOL_PATH} -sv {host}:{port}"
+            command_line = [
+                f"{settings.OLIXTOOL_PATH}",
+                "-sv",
+                f"{host}:{port}",
+            ]
             commands_to_run.append(command_line)
         return commands_to_run
 
